# frozen_string_literal: true

require 'toml'

module GitlabCiRunner
  # Helper methods for configuring gitlab-runner via chef
  module RunnerHelpers
    RUNNER_REGEXP =
      /(docker|parallels|virtualbox|ssh|machine|kubernetes)(_|-)/.freeze

    def feed_options(options_hash, prefix)
      options = []
      options_hash.each do |key, value|
        next if value.is_a?(String) && value.empty?

        if value.is_a?(Hash)
          options.concat(feed_options(value, format_keyname(key.to_s, prefix)))
        else
          options.concat(transform_option(format_keyname(key, prefix), value))
        end
      end
      options
    end

    def transform_option(key, value)
      if value.is_a?(TrueClass) || value.is_a?(FalseClass)
        ["--#{key}=#{value}"]
      elsif key.eql?('tag-list')
        ["--#{key} '#{value.join(',')}'"]
      elsif value.is_a?(Array)
        value.map { |item| "--#{key} '#{item}'" }
      else
        ["--#{key} '#{value}'"]
      end
    end

    def update_runner_options(config:, index:, options:)
      return config if config['runners'].nil? || config['runners'].empty?

      options.each do |option, value|
        next if runner_skip_option?(option) ||
                config['runners'][index][option].eql?(value)

        config['runners'][index][option] = value
      end

      runner_option_cleanup(config: config, index: index, options: options)
    end

    def runner_options_to_h(options)
      options.keys.each do |option|
        next unless runner?(option)

        a = runner_option_split(option)
        options[a[0]] ||= {}
        options[a[0]][a[1]] = options.delete(option)
      end

      options
    end

    def toml_parse(file)
      toml = {}
      toml = ::TOML.load_file(file) if ::File.file?(file)
      toml
    end

    def toml_generate(config_hash)
      ::TOML::Generator.new(config_hash).body
    end

    def runner_token(name, config_file)
      config = toml_parse(config_file)
      config['runners']&.reverse_each do |r|
        return [r['token'], r['url']] if r['name'].eql?(name)
      end

      nil
    end

    def get_runner_index(runners:, url:, token:)
      runners.each_with_index do |runner, i|
        return i if runner['url'].eql?(url) && runner['token'].eql?(token)
      end
    end

    def compact_hash(hash)
      p = proc do |*args|
        v = args.last
        v.delete_if(&p) if v.respond_to? :delete_if
        v.nil? || v.respond_to?(:"empty?") && v.empty?
      end
      hash.delete_if(&p)
    end

    private

    def runner_skip_option?(option)
      %w[name config token registration_token tag_list].include?(option)
    end

    def runner_option_cleanup(config:, index:, options:)
      config['runners'][index].delete_if do |option, _|
        next if runner_skip_option?(option)

        !options.key?(option)
      end

      config
    end

    def runner?(option)
      option.match(RUNNER_REGEXP)
    end

    def runner_option_split(option)
      option.split(RUNNER_REGEXP).delete_if do |string|
        string.empty? || %w[- _].include?(string)
      end
    end

    def format_keyname(key, prefix)
      keyname = key.to_s.tr('_', '-')
      keyname.prepend("#{prefix}-") unless prefix.nil?
      keyname.downcase
    end
  end
end
