# frozen_string_literal: true

#
# Copyright (c) 2015-2017 Sam4Mobile, 2017-2018 Make.org, 2019 Samuel Bernard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# rubocop:disable Metrics/BlockLength

resource_name 'gitlab_ci_runner'

property :description, String
property :url, String, default: 'https://ci.gitlab.com/'
property :config, String, default: '/etc/gitlab-runner/config.toml'
property :unset_proxy, [true, false], default: false
property :sleep, Integer, default: 0
property :tag_list, Array, default: []
property :options, Hash, default: {}
property :register_string, String

action :register do
  extend GitlabCiRunner::RunnerHelpers

  options_hash = {}
  %i[name url config tag_list].each do |key|
    options_hash[key.to_s] = new_resource.send(key)
  end
  options_hash.merge!(new_resource.options.dup)
  options_hash = runner_options_to_h(options_hash)

  if runner_token(new_resource.name, new_resource.config).nil?
    if options_hash.key?('cache')
      log 'Configuration of cache options during intial registration is not '\
          'supported. They will be configured during the next chef run.' do
        level :warn
      end
      options_hash.delete('cache')
    end

    options = if property_is_set?(:register_string)
                new_resource.register_string
              else
                feed_options(options_hash, nil).join(' ')
              end

    env = ''
    if new_resource.unset_proxy
      env = 'http_proxy="" https_proxy="" HTTP_PROXY="" HTTPS_PROXY=""'
    end

    converge_by("Register runner #{new_resource.name}") do
      execute "#{env} gitlab-runner register --non-interactive #{options}"
      # giving time to gitlab-ci to manage the request
      execute "sleep #{new_resource.sleep}"
    end
  else
    Chef::Log.info("Runner #{new_resource.name} already registered.")
    token, url = runner_token(new_resource.name, new_resource.config)
    Chef::Log.info("Found runner with token #{token} and url #{url}.")

    config_hash = toml_parse(new_resource.config)
    runner_index = get_runner_index(
      runners: config_hash['runners'],
      url: url,
      token: token
    )
    Chef::Log.info("Found runner configuration at index #{runner_index}")

    config_hash = update_runner_options(
      config: config_hash,
      index: runner_index,
      options: options_hash
    )
    config_hash = compact_hash(config_hash)

    unless toml_parse(new_resource.config).eql?(config_hash)
      converge_by("Updating config for runner #{new_resource.name}") do
        file new_resource.config do
          content toml_generate(config_hash)
          owner 'root'
          group 'root'
          mode '0600'
          sensitive true
          action :create
          notifies :restart, 'service[gitlab-runner]', :delayed
        end
      end
    end
  end
end

action :unregister do
  extend GitlabCiRunner::RunnerHelpers

  token, url = runner_token(new_resource.name, new_resource.config)

  env = ''
  if new_resource.unset_proxy
    env = 'http_proxy="" https_proxy="" HTTP_PROXY="" HTTPS_PROXY=""'
  end

  unless url.nil? || token.nil?
    converge_by("Unregister runner #{new_resource.description}") do
      execute "#{env} gitlab-runner unregister --token #{token} --url #{url}"
      # giving time to gitlab-ci to manage the request
      execute "sleep #{new_resource.sleep}"
    end
  end
end

# rubocop:enable Metrics/BlockLength
