# frozen_string_literal: true

#
# Cookbook:: gitlab-ci-runner
# Spec:: runner_spec
#
# Copyright:: 2019, Ben Hughes <bmhughes@bmhughes.co.uk>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'spec_helper'

# Disable BlockLength check due to the configuration hashes
# rubocop:disable Metrics/BlockLength

describe 'GitlabCiRunner::RunnerHelpers' do
  let(:dummy_class) { Class.new { include GitlabCiRunner::RunnerHelpers } }
  describe '.feed_options' do
    context 'given options set for register' do
      options = {
        'name': 'Testing Runner',
        'url': 'https://gitlabtest.com',
        'config': '/etc/gitlab-runner/config.toml',
        'registration_token': 'vEs5WVCUGsKwzEzHESED',
        'executor': 'shell',
        'tag_list': [
          'testing-1',
          'testing-2'
        ]
      }

      options_array_cli = [
        "--name 'Testing Runner'",
        "--url 'https://gitlabtest.com'",
        "--config '/etc/gitlab-runner/config.toml'",
        "--registration-token 'vEs5WVCUGsKwzEzHESED'",
        "--executor 'shell'",
        "--tag-list 'testing-1,testing-2'"
      ]

      it 'returns array of cli options' do
        expect(
          dummy_class.new.feed_options(options, nil)
        ).to be_a(Array)
        expect(
          dummy_class.new.feed_options(options, nil)
        ).to eql(options_array_cli)
      end
    end

    context 'given options set with nested hash for register' do
      options = {
        'name': 'Testing Runner',
        'url': 'https://gitlabtest.com',
        'config': '/etc/gitlab-runner/config.toml',
        'registration_token': 'vEs5WVCUGsKwzEzHESED',
        'executor': 'docker',
        'docker': {
          'image': 'centos',
          'privileged': false,
          'disable_entrypoint_overwrite': false,
          'oom_kill_disable': false,
          'disable_cache': false,
          'volumes': [
            '/cache',
            '/root/.chef:/root/.chef:ro',
            '/home/runner/.chef:/home/runner/.chef:ro',
            '/etc/chef:/etc/chef:ro'
          ],
          'shm_size': 0
        },
        'tag_list': [
          'testing-1',
          'testing-2'
        ]
      }

      options_array_cli = [
        "--name 'Testing Runner'",
        "--url 'https://gitlabtest.com'",
        "--config '/etc/gitlab-runner/config.toml'",
        "--registration-token 'vEs5WVCUGsKwzEzHESED'",
        "--executor 'docker'",
        "--docker-image 'centos'",
        '--docker-privileged=false',
        '--docker-disable-entrypoint-overwrite=false',
        '--docker-oom-kill-disable=false',
        '--docker-disable-cache=false',
        "--docker-volumes '/cache'",
        "--docker-volumes '/root/.chef:/root/.chef:ro'",
        "--docker-volumes '/home/runner/.chef:/home/runner/.chef:ro'",
        "--docker-volumes '/etc/chef:/etc/chef:ro'",
        "--docker-shm-size '0'",
        "--tag-list 'testing-1,testing-2'"
      ]

      it 'returns array of cli options' do
        expect(
          dummy_class.new.feed_options(options, nil)
        ).to be_a(Array)
        expect(
          dummy_class.new.feed_options(options, nil)
        ).to eql(options_array_cli)
      end
    end
  end

  describe '.update_runner_options' do
    config = {
      'concurrent' => 4,
      'check_interval' => 0,
      'log_level' => 'info',
      'session_server' => {
        'session_timeout' => 1800
      },
      'runners' => [
        {
          'name' => 'Testing Runner',
          'url' => 'https://gitlab.test.com',
          'token' => 'Ho9KA911jWcTarjsVDti',
          'executor' => 'shell',
          'cache' => {
            's3' => {},
            'gcs' => {}
          }
        },
        {
          'name' => 'Testing Docker Runner',
          'url' => 'https://gitlab.test.com',
          'token' => 'MNzmN6ps5MzGL9zrt4Ce',
          'executor' => 'docker',
          'docker' => {
            'image' => 'centos',
            'privileged' => false,
            'disable_entrypoint_overwrite' => false,
            'oom_kill_disable' => false,
            'disable_cache' => false,
            'volumes' => [
              '/cache',
              '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
              '/etc/chef:/etc/chef:ro'
            ],
            'shm_size' => 0
          },
          'cache' => {
            's3' => {},
            'gcs' => {}
          }
        }
      ]
    }

    options = {
      'url' => 'https://gitlab.test.com',
      'registration_token' => 'vEs5WVCUGsKwzEzHESED',
      'executor' => 'docker',
      'docker' => {
        'image' => 'centos',
        'privileged' => false,
        'disable_entrypoint_overwrite' => false,
        'oom_kill_disable' => false,
        'disable_cache' => false,
        'volumes' => [
          '/cache',
          '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
          '/etc/chef:/etc/chef:ro'
        ],
        'shm_size' => 0
      },
      'cache' => {
        's3' => {},
        'gcs' => {}
      }
    }

    config_changed = {
      'concurrent' => 4,
      'check_interval' => 0,
      'log_level' => 'info',
      'session_server' => {
        'session_timeout' => 1800
      },
      'runners' => [
        {
          'name' => 'Testing Runner',
          'url' => 'https://gitlab.test.com',
          'token' => 'Ho9KA911jWcTarjsVDti',
          'executor' => 'shell',
          'cache' => {
            's3' => {},
            'gcs' => {}
          }
        },
        {
          'name' => 'Testing Docker Runner',
          'url' => 'https://gitlab.test.com',
          'token' => 'MNzmN6ps5MzGL9zrt4Ce',
          'executor' => 'docker',
          'docker' => {
            'image' => 'centos',
            'privileged' => true,
            'disable_entrypoint_overwrite' => false,
            'oom_kill_disable' => false,
            'disable_cache' => false,
            'volumes' => [
              '/cache',
              '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
              '/etc/chef:/etc/chef:ro'
            ],
            'shm_size' => 0
          },
          'cache' => {
            's3' => {
              'ServerAddress' => 's3.amazonaws.com',
              'AccessKey' => 'AMAZON_S3_ACCESS_KEY',
              'SecretKey' => 'AMAZON_S3_SECRET_KEY',
              'BucketName' => 'runners-cache',
              'BucketLocation' => 'eu-west-1',
              'Insecure' => false
            },
            'gcs' => {}
          }
        }
      ]
    }

    options_changed = {
      'url' => 'https://gitlab.test.com',
      'registration_token' => 'vEs5WVCUGsKwzEzHESED',
      'executor' => 'docker',
      'docker' => {
        'image' => 'centos',
        'privileged' => true,
        'disable_entrypoint_overwrite' => false,
        'oom_kill_disable' => false,
        'disable_cache' => false,
        'volumes' => [
          '/cache',
          '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
          '/etc/chef:/etc/chef:ro'
        ],
        'shm_size' => 0
      },
      'cache' => {
        's3' => {
          'ServerAddress' => 's3.amazonaws.com',
          'AccessKey' => 'AMAZON_S3_ACCESS_KEY',
          'SecretKey' => 'AMAZON_S3_SECRET_KEY',
          'BucketName' => 'runners-cache',
          'BucketLocation' => 'eu-west-1',
          'Insecure' => false
        },
        'gcs' => {}
      }
    }

    context('given unchanged options') do
      it 'returns unchanged hash of options' do
        expect(
          dummy_class.new.update_runner_options(
            config: config,
            index: 1,
            options: options
          )
        ).to be_a(Hash)
        expect(
          dummy_class.new.update_runner_options(
            config: config,
            index: 1,
            options: options
          )
        ).to eql(config)
      end
    end

    context 'given changed options' do
      it 'returns changed hash of options' do
        expect(
          dummy_class.new.update_runner_options(
            config: config,
            index: 1,
            options: options_changed
          )
        ).to be_a(Hash)
        expect(
          dummy_class.new.update_runner_options(
            config: config,
            index: 1,
            options: options_changed
          )
        ).to eql(config_changed)
      end
    end
  end

  describe '.runner_options_to_h' do
    config_hash = {
      'url' => 'https://gitlab.test.com',
      'registration_token' => '1234567890',
      'executor' => 'docker',
      'docker' => {
        'image' => 'centos',
        'privileged' => false,
        'disable_entrypoint_overwrite' => false,
        'oom_kill_disable' => false,
        'disable_cache' => false,
        'volumes' => [
          '/cache',
          '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
          '/etc/chef:/etc/chef:ro'
        ],
        'shm_size' => 0
      },
      'cache' => {
        's3' => {},
        'gcs' => {}
      }
    }

    config_string = {
      'url' => 'https://gitlab.test.com',
      'registration_token' => '1234567890',
      'executor' => 'docker',
      'docker_image' => 'centos',
      'docker_privileged' => false,
      'docker_disable_entrypoint_overwrite' => false,
      'docker_oom_kill_disable' => false,
      'docker_disable_cache' => false,
      'docker_volumes' => [
        '/cache',
        '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
        '/etc/chef:/etc/chef:ro'
      ],
      'docker_shm_size' => 0,
      'cache' => {
        's3' => {},
        'gcs' => {}
      }
    }

    context 'given runner config as hash' do
      it 'returns same config hash' do
        expect(
          dummy_class.new.runner_options_to_h(config_hash)
        ).to be_a(Hash)
        expect(
          dummy_class.new.runner_options_to_h(config_hash)
        ).to eql(config_hash)
      end
    end

    context 'given runner config as string' do
      it 'returns config converted to hash' do
        expect(
          dummy_class.new.runner_options_to_h(config_string)
        ).to be_a(Hash)
        expect(
          dummy_class.new.runner_options_to_h(config_string)
        ).to eql(config_hash)
      end
    end
  end

  describe '.get_runner_index' do
    config_toml = {
      'concurrent' => 5, 'check_interval' => 3,
      'session_server' => {
        'session_timeout' => 1800
      },
      'runners' => [
        {
          'name' => 'Testing Shell Runner',
          'url' => 'https://gitlab.test.com',
          'token' => '1234567890',
          'executor' => 'shell',
          'custom_build_dir' => {},
          'cache' => {
            's3' => {},
            'gcs' => {}
          }
        },
        {
          'name' => 'Testing Docker Runner',
          'url' => 'https://gitlab.test.com',
          'token' => '9876543210',
          'executor' => 'docker',
          'custom_build_dir' => {},
          'docker' => {
            'tls_verify' => false,
            'image' => 'centos',
            'privileged' => false,
            'disable_entrypoint_overwrite' => false,
            'oom_kill_disable' => false,
            'disable_cache' => false,
            'volumes' => [
              '/cache', '/root/.chef:/root/.chef:ro',
              '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
              '/etc/chef:/etc/chef:ro'
            ],
            'shm_size' => 0
          },
          'cache' => {
            's3' => {},
            'gcs' => {}
          }
        }
      ]
    }

    context 'given runner config as string' do
      it 'returns array index of specified runner' do
        expect(
          dummy_class.new.get_runner_index(
            runners: config_toml['runners'],
            url: 'https://gitlab.test.com',
            token: '1234567890'
          )
        ).to be_a(Integer)
        expect(
          dummy_class.new.get_runner_index(
            runners: config_toml['runners'],
            url: 'https://gitlab.test.com',
            token: '1234567890'
          )
        ).to eql(0)
        expect(
          dummy_class.new.get_runner_index(
            runners: config_toml['runners'],
            url: 'https://gitlab.test.com',
            token: '9876543210'
          )
        ).to eql(1)
      end
    end
  end

  describe '.runner_token' do
    before(:each) do
      allow(File).to receive(:file?).and_call_original
      allow(TOML).to receive(:load_file).and_call_original
    end

    context 'given runner config as string' do
      config_toml = {
        'concurrent' => 5, 'check_interval' => 3,
        'session_server' => {
          'session_timeout' => 1800
        },
        'runners' => [
          {
            'name' => 'Testing Shell Runner',
            'url' => 'https://gitlab.test.com',
            'token' => '1234567890',
            'executor' => 'shell',
            'custom_build_dir' => {},
            'cache' => {
              's3' => {},
              'gcs' => {}
            }
          },
          {
            'name' => 'Testing Docker Runner',
            'url' => 'https://gitlab.test.com',
            'token' => '9876543210',
            'executor' => 'docker',
            'custom_build_dir' => {},
            'docker' => {
              'tls_verify' => false,
              'image' => 'centos',
              'privileged' => false,
              'disable_entrypoint_overwrite' => false,
              'oom_kill_disable' => false,
              'disable_cache' => false,
              'volumes' => [
                '/cache', '/root/.chef:/root/.chef:ro',
                '/home/gitlab-runner/.chef:/home/gitlab-runner/.chef:ro',
                '/etc/chef:/etc/chef:ro'
              ],
              'shm_size' => 0
            },
            'cache' => {
              's3' => {},
              'gcs' => {}
            }
          }
        ]
      }

      it 'returns array index of specified runner' do
        allow(File).to receive(:file?).with(/toml/).and_return(true)
        allow(TOML).to receive(:load_file).with(/toml/).and_return(config_toml)

        expect(
          dummy_class.new.runner_token(
            'Testing Shell Runner',
            '/etc/gitlab-runner/config.toml'
          )
        ).to be_a(Array)
        expect(
          dummy_class.new.runner_token(
            'Testing Shell Runner',
            '/etc/gitlab-runner/config.toml'
          )
        ).to eql(['1234567890', 'https://gitlab.test.com'])
      end
    end
  end

  describe '.toml_generate' do
    test_hash = {
      'test' => 'hash',
      'subhash' => {
        'one' => 1,
        'two' => 'two',
        'three' => 3.0
      }
    }

    str = "test = \"hash\"\n\n[subhash]\none = 1\nthree = 3.0\ntwo = \"two\"\n"
    context 'given configuration hash' do
      it 'returns string output of toml file' do
        expect(
          dummy_class.new.toml_generate(test_hash)
        ).to be_a(String)

        expect(
          dummy_class.new.toml_generate(test_hash)
        ).to eql(str)
      end
    end
  end

  describe '.compact_hash' do
    hash_with_empty = {
      'a' => 1,
      'b' => {},
      'c' => 3
    }

    hash_with_empty_result = {
      'a' => 1,
      'c' => 3
    }

    hash_with_nil = {
      'a' => 1,
      'b' => 2,
      'c' => nil
    }

    hash_with_nil_result = {
      'a' => 1,
      'b' => 2
    }

    context 'given hash with empty values' do
      it 'removes them' do
        expect(
          dummy_class.new.compact_hash(hash_with_empty)
        ).to eql(hash_with_empty_result)
      end
    end

    context 'given hash with nil values' do
      it 'removes them' do
        expect(
          dummy_class.new.compact_hash(hash_with_nil)
        ).to eql(hash_with_nil_result)
      end
    end
  end
end

# rubocop:enable Metrics/BlockLength
